@@ -1,4 +1,4 @@
-/* $Id: attributes.c 2553 2011-07-09 09:42:33Z tcunha $ */
+/* $OpenBSD$ */
 
 /*
  * Copyright (c) 2009 Joshua Elsasser <josh@elsasser.org>
@@ -23,30 +23,25 @@
 #include "tmux.h"
 
 const char *
-attributes_tostring(u_char attr)
+attributes_tostring(int attr)
 {
 	static char	buf[128];
+	size_t		len;
 
 	if (attr == 0)
 		return ("none");
 
-	buf[0] = '\0';
-	if (attr & GRID_ATTR_BRIGHT)
-		strlcat(buf, "bright,", sizeof (buf));
-	if (attr & GRID_ATTR_DIM)
-		strlcat(buf, "dim,", sizeof (buf));
-	if (attr & GRID_ATTR_UNDERSCORE)
-		strlcat(buf, "underscore,", sizeof (buf));
-	if (attr & GRID_ATTR_BLINK)
-		strlcat(buf, "blink,", sizeof (buf));
-	if (attr & GRID_ATTR_REVERSE)
-		strlcat(buf, "reverse,", sizeof (buf));
-	if (attr & GRID_ATTR_HIDDEN)
-		strlcat(buf, "hidden,", sizeof (buf));
-	if (attr & GRID_ATTR_ITALICS)
-		strlcat(buf, "italics,", sizeof (buf));
-	if (*buf != '\0')
-		*(strrchr(buf, ',')) = '\0';
+	len = xsnprintf(buf, sizeof buf, "%s%s%s%s%s%s%s%s",
+	    (attr & GRID_ATTR_BRIGHT) ? "bright," : "",
+	    (attr & GRID_ATTR_DIM) ? "dim," : "",
+	    (attr & GRID_ATTR_UNDERSCORE) ? "underscore," : "",
+	    (attr & GRID_ATTR_BLINK)? "blink," : "",
+	    (attr & GRID_ATTR_REVERSE) ? "reverse," : "",
+	    (attr & GRID_ATTR_HIDDEN) ? "hidden," : "",
+	    (attr & GRID_ATTR_ITALICS) ? "italics," : "",
+	    (attr & GRID_ATTR_STRIKETHROUGH) ? "strikethrough," : "");
+	if (len > 0)
+		buf[len - 1] = '\0';
 
 	return (buf);
 }
@@ -55,7 +50,7 @@ int
 attributes_fromstring(const char *str)
 {
 	const char	delimiters[] = " ,|";
-	u_char		attr;
+	int		attr;
 	size_t		end;
 
 	if (*str == '\0' || strcspn(str, delimiters) == 0)
@@ -84,6 +79,8 @@ attributes_fromstring(const char *str)
 			attr |= GRID_ATTR_HIDDEN;
 		else if (end == 7 && strncasecmp(str, "italics", end) == 0)
 			attr |= GRID_ATTR_ITALICS;
+		else if (end == 13 && strncasecmp(str, "strikethrough", end) == 0)
+			attr |= GRID_ATTR_STRIKETHROUGH;
 		else
 			return (-1);
 		str += end + strspn(str + end, delimiters);
