@@ -241,7 +241,7 @@ cmdq_fire_command(struct cmdq_item *item)
 			fsp = &item->target;
 		else if (cmd_find_valid_state(&item->shared->current))
 			fsp = &item->shared->current;
-		else if (cmd_find_from_client(&fs, item->client) == 0)
+		else if (cmd_find_from_client(&fs, item->client, 0) == 0)
 			fsp = &fs;
 		else
 			goto out;
@@ -428,7 +428,8 @@ cmdq_print(struct cmdq_item *item, const char *fmt, ...)
 		w = c->session->curw->window;
 		if (w->active->mode != &window_copy_mode) {
 			window_pane_reset_mode(w->active);
-			window_pane_set_mode(w->active, &window_copy_mode);
+			window_pane_set_mode(w->active, &window_copy_mode, NULL,
+			    NULL);
 			window_copy_init_for_output(w->active);
 		}
 		window_copy_vadd(w->active, fmt, ap);
@@ -452,6 +453,8 @@ cmdq_error(struct cmdq_item *item, const char *fmt, ...)
 	msglen = xvasprintf(&msg, fmt, ap);
 	va_end(ap);
 
+	log_debug("%s: %s", __func__, msg);
+
 	if (c == NULL)
 		cfg_add_cause("%s:%u: %s", cmd->file, cmd->line, msg);
 	else if (c->session == NULL || (c->flags & CLIENT_CONTROL)) {
