@@ -1,7 +1,7 @@
 /* $OpenBSD$ */
 
 /*
- * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
+ * Copyright (c) 2008 Nicholas Marriott <nicholas.marriott@gmail.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -31,14 +31,14 @@
  * Parse a command from a string.
  */
 
-int	 cmd_string_getc(const char *, size_t *);
-void	 cmd_string_ungetc(size_t *);
-void	 cmd_string_copy(char **, char *, size_t *);
-char	*cmd_string_string(const char *, size_t *, char, int);
-char	*cmd_string_variable(const char *, size_t *);
-char	*cmd_string_expand_tilde(const char *, size_t *);
+static int	 cmd_string_getc(const char *, size_t *);
+static void	 cmd_string_ungetc(size_t *);
+static void	 cmd_string_copy(char **, char *, size_t *);
+static char	*cmd_string_string(const char *, size_t *, char, int);
+static char	*cmd_string_variable(const char *, size_t *);
+static char	*cmd_string_expand_tilde(const char *, size_t *);
 
-int
+static int
 cmd_string_getc(const char *s, size_t *p)
 {
 	const u_char	*ucs = s;
@@ -48,38 +48,21 @@ cmd_string_getc(const char *s, size_t *p)
 	return (ucs[(*p)++]);
 }
 
-void
+static void
 cmd_string_ungetc(size_t *p)
 {
 	(*p)--;
 }
 
-/*
- * Parse command string. Returns -1 on error. If returning -1, cause is error
- * string, or NULL for empty command.
- */
 int
-cmd_string_parse(const char *s, struct cmd_list **cmdlist, const char *file,
-    u_int line, char **cause)
+cmd_string_split(const char *s, int *rargc, char ***rargv)
 {
-	size_t		p;
-	int		ch, i, argc, rval;
-	char	      **argv, *buf, *t;
+	size_t		p = 0;
+	int		ch, argc = 0, append = 0;
+	char	      **argv = NULL, *buf = NULL, *t;
 	const char     *whitespace, *equals;
-	size_t		len;
+	size_t		len = 0;
 
-	argv = NULL;
-	argc = 0;
-
-	buf = NULL;
-	len = 0;
-
-	*cause = NULL;
-
-	*cmdlist = NULL;
-	rval = -1;
-
-	p = 0;
 	for (;;) {
 		ch = cmd_string_getc(s, &p);
 		switch (ch) {
@@ -126,54 +109,72 @@ cmd_string_parse(const char *s, struct cmd_list **cmdlist, const char *file,
 				whitespace = argv[0] + strcspn(argv[0], " \t");
 				if (equals == NULL || equals > whitespace)
 					break;
-				environ_put(&global_environ, argv[0]);
+				environ_put(global_environ, argv[0]);
 				argc--;
 				memmove(argv, argv + 1, argc * (sizeof *argv));
 			}
-			if (argc == 0)
-				goto out;
-
-			*cmdlist = cmd_list_parse(argc, argv, file, line, cause);
-			if (*cmdlist == NULL)
-				goto out;
-
-			rval = 0;
-			goto out;
+			goto done;
 		case '~':
-			if (buf == NULL) {
-				t = cmd_string_expand_tilde(s, &p);
-				if (t == NULL)
-					goto error;
-				cmd_string_copy(&buf, t, &len);
+			if (buf != NULL) {
+				append = 1;
 				break;
 			}
-			/* FALLTHROUGH */
-		default:
-			if (len >= SIZE_MAX - 2)
+			t = cmd_string_expand_tilde(s, &p);
+			if (t == NULL)
 				goto error;
-
-			buf = xrealloc(buf, len + 1);
-			buf[len++] = ch;
+			cmd_string_copy(&buf, t, &len);
+			break;
+		default:
+			append = 1;
 			break;
 		}
+		if (append) {
+			if (len >= SIZE_MAX - 2)
+				goto error;
+			buf = xrealloc(buf, len + 1);
+			buf[len++] = ch;
+		}
+		append = 0;
 	}
 
+done:
+	*rargc = argc;
+	*rargv = argv;
+
+	free(buf);
+	return (0);
+
 error:
-	xasprintf(cause, "invalid or unknown command: %s", s);
-
-out:
+	if (argv != NULL)
+		cmd_free_argv(argc, argv);
 	free(buf);
-
-	if (argv != NULL) {
-		for (i = 0; i < argc; i++)
-			free(argv[i]);
-		free(argv);
-	}
-
-	return (rval);
+	return (-1);
 }
 
-void
+struct cmd_list *
+cmd_string_parse(const char *s, const char *file, u_int line, char **cause)
+{
+	struct cmd_list	 *cmdlist = NULL;
+	int		  argc;
+	char		**argv;
+
+	*cause = NULL;
+	if (cmd_string_split(s, &argc, &argv) != 0) {
+		xasprintf(cause, "invalid or unknown command: %s", s);
+		return (NULL);
+	}
+	if (argc != 0) {
+		cmdlist = cmd_list_parse(argc, argv, file, line, cause);
+		if (cmdlist == NULL) {
+			cmd_free_argv(argc, argv);
+			return (NULL);
+		}
+	}
+	cmd_free_argv(argc, argv);
+	return (cmdlist);
+}
+
+static void
 cmd_string_copy(char **dst, char *src, size_t *len)
 {
 	size_t srclen;
@@ -187,7 +188,7 @@ cmd_string_copy(char **dst, char *src, size_t *len)
 	free(src);
 }
 
-char *
+static char *
 cmd_string_string(const char *s, size_t *p, char endch, int esc)
 {
 	int	ch;
@@ -245,7 +246,7 @@ error:
 	return (NULL);
 }
 
-char *
+static char *
 cmd_string_variable(const char *s, size_t *p)
 {
 	int			ch, fch;
@@ -303,7 +304,7 @@ cmd_string_variable(const char *s, size_t *p)
 	buf = xrealloc(buf, len + 1);
 	buf[len] = '\0';
 
-	envent = environ_find(&global_environ, buf);
+	envent = environ_find(global_environ, buf);
 	free(buf);
 	if (envent == NULL)
 		return (xstrdup(""));
@@ -314,7 +315,7 @@ error:
 	return (NULL);
 }
 
-char *
+static char *
 cmd_string_expand_tilde(const char *s, size_t *p)
 {
 	struct passwd		*pw;
@@ -326,7 +327,7 @@ cmd_string_expand_tilde(const char *s, size_t *p)
 
 	last = cmd_string_getc(s, p);
 	if (last == EOF || last == '/' || last == ' '|| last == '\t') {
-		envent = environ_find(&global_environ, "HOME");
+		envent = environ_find(global_environ, "HOME");
 		if (envent != NULL && *envent->value != '\0')
 			home = envent->value;
 		else if ((pw = getpwuid(getuid())) != NULL)
@@ -337,7 +338,10 @@ cmd_string_expand_tilde(const char *s, size_t *p)
 		cp = user = xmalloc(strlen(s));
 		for (;;) {
 			last = cmd_string_getc(s, p);
-			if (last == EOF || last == '/' || last == ' '|| last == '\t')
+			if (last == EOF ||
+			    last == '/' ||
+			    last == ' '||
+			    last == '\t')
 				break;
 			*cp++ = last;
 		}
