@@ -1,7 +1,5 @@
-/* $Id: asprintf.c 2567 2011-07-25 09:56:43Z nicm $ */
-
 /*
- * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
+ * Copyright (c) 2006 Nicholas Marriott <nicholas.marriott@gmail.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -16,16 +14,13 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include <sys/types.h>
+
 #include <stdarg.h>
 #include <stdio.h>
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#else
-#include <inttypes.h>
-#endif
 #include <string.h>
 
-#include "tmux.h"
+#include "compat.h"
 
 int
 asprintf(char **ret, const char *fmt, ...)
@@ -53,13 +48,15 @@ vasprintf(char **ret, const char *fmt, va_list ap)
 
 	*ret = xmalloc(n + 1);
 	if ((n = vsnprintf(*ret, n + 1, fmt, ap2)) < 0) {
-		xfree(*ret);
+		free(*ret);
 		goto error;
 	}
+	va_end(ap2);
 
 	return (n);
 
 error:
+	va_end(ap2);
 	*ret = NULL;
 	return (-1);
 }
