@@ -17,13 +17,11 @@
  */
 
 #include <sys/types.h>
-#include <sys/param.h>
 #include <sys/wait.h>
 
-#include <ctype.h>
 #include <errno.h>
+#include <fnmatch.h>
 #include <libgen.h>
-#include <netdb.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
@@ -43,28 +41,6 @@ typedef void (*format_cb)(struct format_tree *, struct format_entry *);
 static char	*format_job_get(struct format_tree *, const char *);
 static void	 format_job_timer(int, short, void *);
 
-static void	 format_cb_host(struct format_tree *, struct format_entry *);
-static void	 format_cb_host_short(struct format_tree *,
-		     struct format_entry *);
-static void	 format_cb_pid(struct format_tree *, struct format_entry *);
-static void	 format_cb_session_alerts(struct format_tree *,
-		     struct format_entry *);
-static void	 format_cb_window_layout(struct format_tree *,
-		     struct format_entry *);
-static void	 format_cb_window_visible_layout(struct format_tree *,
-		     struct format_entry *);
-static void	 format_cb_start_command(struct format_tree *,
-		     struct format_entry *);
-static void	 format_cb_current_command(struct format_tree *,
-		     struct format_entry *);
-static void	 format_cb_history_bytes(struct format_tree *,
-		     struct format_entry *);
-static void	 format_cb_pane_tabs(struct format_tree *,
-		     struct format_entry *);
-
-static void	 format_cb_current_path(struct format_tree *,
-		     struct format_entry *);
-
 static char	*format_find(struct format_tree *, const char *, int);
 static void	 format_add_cb(struct format_tree *, const char *, format_cb);
 static void	 format_add_tv(struct format_tree *, const char *,
@@ -217,7 +193,6 @@ format_job_update(struct job *job)
 	struct format_job	*fj = job->data;
 	char			*line;
 	time_t			 t;
-	struct client		*c;
 
 	if ((line = evbuffer_readline(job->event->input)) == NULL)
 		return;
@@ -226,12 +201,12 @@ format_job_update(struct job *job)
 	free(fj->out);
 	fj->out = line;
 
-	log_debug("%s: %s: %s", __func__, fj->cmd, fj->out);
+	log_debug("%s: %p %s: %s", __func__, fj, fj->cmd, fj->out);
 
-	t = time (NULL);
+	t = time(NULL);
 	if (fj->status && fj->last != t) {
-		TAILQ_FOREACH(c, &clients, entry)
-		    server_status_client(c);
+		if (fj->client != NULL)
+			server_status_client(fj->client);
 		fj->last = t;
 	}
 }
@@ -256,10 +231,11 @@ format_job_complete(struct job *job)
 	} else
 		buf = line;
 
+	log_debug("%s: %p %s: %s", __func__, fj, fj->cmd, buf);
+
 	if (*buf != '\0' || !fj->updated) {
 		free(fj->out);
 		fj->out = buf;
-		log_debug("%s: %s: %s", __func__, fj->cmd, fj->out);
 	} else
 		free(buf);
 
@@ -551,7 +527,7 @@ format_cb_current_command(struct format_tree *ft, struct format_entry *fe)
 }
 
 /* Callback for pane_current_path. */
-void
+static void
 format_cb_current_path(struct format_tree *ft, struct format_entry *fe)
 {
 	struct window_pane	*wp = ft->wp;
@@ -878,7 +854,7 @@ format_choose(char *s, char **left, char **right)
 }
 
 /* Is this true? */
-static int
+int
 format_true(const char *s)
 {
 	if (s != NULL && *s != '\0' && (s[0] != '0' || s[1] != '\0'))
@@ -886,19 +862,17 @@ format_true(const char *s)
 	return (0);
 }
 
-/*
- * Replace a key/value pair in buffer. #{blah} is expanded directly,
- * #{?blah,a,b} is replace with a if blah exists and is nonzero else b.
- */
+/* Replace a key. */
 static int
 format_replace(struct format_tree *ft, const char *key, size_t keylen,
     char **buf, size_t *len, size_t *off)
 {
-	char		*copy, *copy0, *endptr, *ptr, *found, *new, *value;
-	char		*from = NULL, *to = NULL, *left, *right;
-	size_t		 valuelen, newlen, fromlen, tolen, used;
-	long		 limit = 0;
-	int		 modifiers = 0, compare = 0;
+	struct window_pane	*wp = ft->wp;
+	char			*copy, *copy0, *endptr, *ptr, *found, *new;
+	char			*value, *from = NULL, *to = NULL, *left, *right;
+	size_t			 valuelen, newlen, fromlen, tolen, used;
+	long			 limit = 0;
+	int			 modifiers = 0, compare = 0, search = 0;
 
 	/* Make a copy of the key. */
 	copy0 = copy = xmalloc(keylen + 1);
@@ -907,6 +881,30 @@ format_replace(struct format_tree *ft, const char *key, size_t keylen,
 
 	/* Is there a length limit or whatnot? */
 	switch (copy[0]) {
+	case 'm':
+		if (copy[1] != ':')
+			break;
+		compare = -2;
+		copy += 2;
+		break;
+	case 'C':
+		if (copy[1] != ':')
+			break;
+		search = 1;
+		copy += 2;
+		break;
+	case '|':
+		if (copy[1] != '|' || copy[2] != ':')
+			break;
+		compare = -3;
+		copy += 3;
+		break;
+	case '&':
+		if (copy[1] != '&' || copy[2] != ':')
+			break;
+		compare = -4;
+		copy += 3;
+		break;
 	case '!':
 		if (copy[1] == '=' && copy[2] == ':') {
 			compare = -1;
@@ -972,16 +970,30 @@ format_replace(struct format_tree *ft, const char *key, size_t keylen,
 	}
 
 	/* Is this a comparison or a conditional? */
-	if (compare != 0) {
+	if (search) {
+		/* Search in pane. */
+		if (wp == NULL)
+			value = xstrdup("0");
+		else
+			xasprintf(&value, "%u", window_pane_search(wp, copy));
+	} else if (compare != 0) {
 		/* Comparison: compare comma-separated left and right. */
 		if (format_choose(copy, &left, &right) != 0)
 			goto fail;
 		left = format_expand(ft, left);
 		right = format_expand(ft, right);
-		if (compare == 1 && strcmp(left, right) == 0)
+		if (compare == -3 &&
+		    (format_true(left) || format_true(right)))
+			value = xstrdup("1");
+		else if (compare == -4 &&
+		    (format_true(left) && format_true(right)))
+			value = xstrdup("1");
+		else if (compare == 1 && strcmp(left, right) == 0)
 			value = xstrdup("1");
 		else if (compare == -1 && strcmp(left, right) != 0)
 			value = xstrdup("1");
+		else if (compare == -2 && fnmatch(left, right, 0) == 0)
+			value = xstrdup("1");
 		else
 			value = xstrdup("0");
 		free(right);
@@ -1220,6 +1232,10 @@ void
 format_defaults(struct format_tree *ft, struct client *c, struct session *s,
     struct winlink *wl, struct window_pane *wp)
 {
+	format_add(ft, "session_format", "%d", s != NULL);
+	format_add(ft, "window_format", "%d", wl != NULL);
+	format_add(ft, "pane_format", "%d", wp != NULL);
+
 	if (s == NULL && c != NULL)
 		s = c->session;
 	if (wl == NULL && s != NULL)
@@ -1377,7 +1393,7 @@ format_defaults_pane(struct format_tree *ft, struct window_pane *wp)
 {
 	struct grid	*gd = wp->base.grid;
 	u_int		 idx;
-	int  		 status, scroll_position;
+	int  		 status;
 
 	if (ft->w == NULL)
 		ft->w = wp->window;
@@ -1397,6 +1413,7 @@ format_defaults_pane(struct format_tree *ft, struct window_pane *wp)
 	format_add(ft, "pane_id", "%%%u", wp->id);
 	format_add(ft, "pane_active", "%d", wp == wp->window->active);
 	format_add(ft, "pane_input_off", "%d", !!(wp->flags & PANE_INPUTOFF));
+	format_add(ft, "pane_pipe", "%d", wp->pipe_fd != -1);
 
 	status = wp->status;
 	if (wp->fd == -1 && WIFEXITED(status))
@@ -1408,6 +1425,10 @@ format_defaults_pane(struct format_tree *ft, struct window_pane *wp)
 		format_add(ft, "pane_top", "%u", wp->yoff);
 		format_add(ft, "pane_right", "%u", wp->xoff + wp->sx - 1);
 		format_add(ft, "pane_bottom", "%u", wp->yoff + wp->sy - 1);
+		format_add(ft, "pane_at_left", "%d", wp->xoff == 0);
+		format_add(ft, "pane_at_top", "%d", wp->yoff == 0);
+		format_add(ft, "pane_at_right", "%d", wp->xoff + wp->sx == wp->window->sx);
+		format_add(ft, "pane_at_bottom", "%d", wp->yoff + wp->sy == wp->window->sy);
 	}
 
 	format_add(ft, "pane_in_mode", "%d", wp->screen != &wp->base);
@@ -1416,8 +1437,8 @@ format_defaults_pane(struct format_tree *ft, struct window_pane *wp)
 
 	format_add(ft, "pane_synchronized", "%d",
 	    !!options_get_number(wp->window->options, "synchronize-panes"));
-	format_add(ft, "pane_search_string", "%s",
-	    window_copy_search_string(wp));
+	if (wp->searchstr != NULL)
+		format_add(ft, "pane_search_string", "%s", wp->searchstr);
 
 	format_add(ft, "pane_tty", "%s", wp->tty);
 	format_add(ft, "pane_pid", "%ld", (long) wp->pid);
@@ -1430,9 +1451,7 @@ format_defaults_pane(struct format_tree *ft, struct window_pane *wp)
 	format_add(ft, "scroll_region_upper", "%u", wp->base.rupper);
 	format_add(ft, "scroll_region_lower", "%u", wp->base.rlower);
 
-	scroll_position = window_copy_scroll_position(wp);
-	if (scroll_position != -1)
-		format_add(ft, "scroll_position", "%d", scroll_position);
+	window_copy_add_formats(wp, ft);
 
 	format_add(ft, "alternate_on", "%d", wp->saved_grid ? 1 : 0);
 	format_add(ft, "alternate_saved_x", "%u", wp->saved_cx);
@@ -1465,12 +1484,17 @@ format_defaults_pane(struct format_tree *ft, struct window_pane *wp)
 void
 format_defaults_paste_buffer(struct format_tree *ft, struct paste_buffer *pb)
 {
-	size_t	 bufsize;
-	char	*s;
+	struct timeval	 tv;
+	size_t		 size;
+	char		*s;
 
-	paste_buffer_data(pb, &bufsize);
-	format_add(ft, "buffer_size", "%zu", bufsize);
+	timerclear(&tv);
+	tv.tv_sec = paste_buffer_created(pb);
+	paste_buffer_data(pb, &size);
+
+	format_add(ft, "buffer_size", "%zu", size);
 	format_add(ft, "buffer_name", "%s", paste_buffer_name(pb));
+	format_add_tv(ft, "buffer_created", &tv);
 
 	s = paste_make_sample(pb);
 	format_add(ft, "buffer_sample", "%s", s);
