@@ -94,6 +94,48 @@ grid_need_extended_cell(const struct grid_cell_entry *gce,
 	return (0);
 }
 
+/* Free up unused extended cells. */
+static void
+grid_compact_line(struct grid_line *gl)
+{
+	int			 new_extdsize = 0;
+	struct grid_cell	*new_extddata;
+	struct grid_cell_entry	*gce;
+	struct grid_cell	*gc;
+	u_int			 px, idx;
+
+	if (gl->extdsize == 0)
+		return;
+
+	for (px = 0; px < gl->cellsize; px++) {
+		gce = &gl->celldata[px];
+		if (gce->flags & GRID_FLAG_EXTENDED)
+			new_extdsize++;
+	}
+
+	if (new_extdsize == 0) {
+		free(gl->extddata);
+		gl->extddata = NULL;
+		gl->extdsize = 0;
+		return;
+	}
+	new_extddata = xreallocarray(NULL, new_extdsize, sizeof *gl->extddata);
+
+	idx = 0;
+	for (px = 0; px < gl->cellsize; px++) {
+		gce = &gl->celldata[px];
+		if (gce->flags & GRID_FLAG_EXTENDED) {
+			gc = &gl->extddata[gce->offset];
+			memcpy(&new_extddata[idx], gc, sizeof *gc);
+			gce->offset = idx++;
+		}
+	}
+
+	free(gl->extddata);
+	gl->extddata = new_extddata;
+	gl->extdsize = new_extdsize;
+}
+
 /* Set cell as extended. */
 static struct grid_cell *
 grid_extended_cell(struct grid_line *gl, struct grid_cell_entry *gce,
@@ -162,6 +204,26 @@ grid_cells_equal(const struct grid_cell *gca, const struct grid_cell *gcb)
 	return (memcmp(gca->data.data, gcb->data.data, gca->data.size) == 0);
 }
 
+/* Free one line. */
+static void
+grid_free_line(struct grid *gd, u_int py)
+{
+	free(gd->linedata[py].celldata);
+	gd->linedata[py].celldata = NULL;
+	free(gd->linedata[py].extddata);
+	gd->linedata[py].extddata = NULL;
+}
+
+/* Free several lines. */
+static void
+grid_free_lines(struct grid *gd, u_int py, u_int ny)
+{
+	u_int	yy;
+
+	for (yy = py; yy < py + ny; yy++)
+		grid_free_line(gd, yy);
+}
+
 /* Create a new grid. */
 struct grid *
 grid_create(u_int sx, u_int sy, u_int hlimit)
@@ -187,14 +249,7 @@ grid_create(u_int sx, u_int sy, u_int hlimit)
 void
 grid_destroy(struct grid *gd)
 {
-	struct grid_line	*gl;
-	u_int			 yy;
-
-	for (yy = 0; yy < gd->hsize + gd->sy; yy++) {
-		gl = &gd->linedata[yy];
-		free(gl->celldata);
-		free(gl->extddata);
-	}
+	grid_free_lines(gd, 0, gd->hsize + gd->sy);
 
 	free(gd->linedata);
 
@@ -233,19 +288,28 @@ grid_compare(struct grid *ga, struct grid *gb)
  * and shift up.
  */
 void
-grid_collect_history(struct grid *gd, u_int bg)
+grid_collect_history(struct grid *gd)
 {
-	u_int	yy;
+	u_int	ny;
 
-	if (gd->hsize < gd->hlimit)
+	if (gd->hsize == 0 || gd->hsize < gd->hlimit)
 		return;
 
-	yy = gd->hlimit / 10;
-	if (yy < 1)
-		yy = 1;
+	ny = gd->hlimit / 10;
+	if (ny < 1)
+		ny = 1;
+	if (ny > gd->hsize)
+		ny = gd->hsize;
 
-	grid_move_lines(gd, 0, yy, gd->hsize + gd->sy - yy, bg);
-	gd->hsize -= yy;
+	/*
+	 * Free the lines from 0 to ny then move the remaining lines over
+	 * them.
+	 */
+	grid_free_lines(gd, 0, ny);
+	memmove(&gd->linedata[0], &gd->linedata[ny],
+	    (gd->hsize + gd->sy - ny) * (sizeof *gd->linedata));
+
+	gd->hsize -= ny;
 	if (gd->hscrolled > gd->hsize)
 		gd->hscrolled = gd->hsize;
 }
@@ -265,6 +329,7 @@ grid_scroll_history(struct grid *gd, u_int bg)
 	grid_empty_line(gd, yy, bg);
 
 	gd->hscrolled++;
+	grid_compact_line(&gd->linedata[gd->hsize]);
 	gd->hsize++;
 }
 
@@ -272,8 +337,9 @@ grid_scroll_history(struct grid *gd, u_int bg)
 void
 grid_clear_history(struct grid *gd)
 {
-	grid_clear_lines(gd, 0, gd->hsize, 8);
-	grid_move_lines(gd, 0, gd->hsize, gd->sy, 8);
+	grid_free_lines(gd, 0, gd->hsize);
+	memmove(&gd->linedata[0], &gd->linedata[gd->hsize],
+	    gd->sy * (sizeof *gd->linedata));
 
 	gd->hscrolled = 0;
 	gd->hsize = 0;
@@ -284,9 +350,9 @@ grid_clear_history(struct grid *gd)
 
 /* Scroll a region up, moving the top line into the history. */
 void
-grid_scroll_history_region(struct grid *gd, u_int upper, u_int lower)
+grid_scroll_history_region(struct grid *gd, u_int upper, u_int lower, u_int bg)
 {
-	struct grid_line	*gl_history, *gl_upper, *gl_lower;
+	struct grid_line	*gl_history, *gl_upper;
 	u_int			 yy;
 
 	/* Create a space for a new line. */
@@ -302,14 +368,13 @@ grid_scroll_history_region(struct grid *gd, u_int upper, u_int lower)
 	upper++;
 	gl_upper = &gd->linedata[upper];
 	lower++;
-	gl_lower = &gd->linedata[lower];
 
 	/* Move the line into the history. */
 	memcpy(gl_history, gl_upper, sizeof *gl_history);
 
 	/* Then move the region up and clear the bottom line. */
 	memmove(gl_upper, gl_upper + 1, (lower - upper) * sizeof *gl_upper);
-	memset(gl_lower, 0, sizeof *gl_lower);
+	grid_empty_line(gd, lower, bg);
 
 	/* Move the history offset down over the line. */
 	gd->hscrolled++;
@@ -482,8 +547,7 @@ grid_clear(struct grid *gd, u_int px, u_int py, u_int nx, u_int ny, u_int bg)
 void
 grid_clear_lines(struct grid *gd, u_int py, u_int ny, u_int bg)
 {
-	struct grid_line	*gl;
-	u_int			 yy;
+	u_int	yy;
 
 	if (ny == 0)
 		return;
@@ -494,9 +558,7 @@ grid_clear_lines(struct grid *gd, u_int py, u_int ny, u_int bg)
 		return;
 
 	for (yy = py; yy < py + ny; yy++) {
-		gl = &gd->linedata[yy];
-		free(gl->celldata);
-		free(gl->extddata);
+		grid_free_line(gd, yy);
 		grid_empty_line(gd, yy, bg);
 	}
 }
@@ -523,13 +585,16 @@ grid_move_lines(struct grid *gd, u_int dy, u_int py, u_int ny, u_int bg)
 	for (yy = dy; yy < dy + ny; yy++) {
 		if (yy >= py && yy < py + ny)
 			continue;
-		grid_clear_lines(gd, yy, 1, bg);
+		grid_free_line(gd, yy);
 	}
 
 	memmove(&gd->linedata[dy], &gd->linedata[py],
 	    ny * (sizeof *gd->linedata));
 
-	/* Wipe any lines that have been moved (without freeing them). */
+	/*
+	 * Wipe any lines that have been moved (without freeing them - they are
+	 * still present).
+	 */
 	for (yy = py; yy < py + ny; yy++) {
 		if (yy < dy || yy >= dy + ny)
 			grid_empty_line(gd, yy, bg);
@@ -666,8 +731,7 @@ grid_string_cells_bg(const struct grid_cell *gc, int *values)
 
 /*
  * Returns ANSI code to set particular attributes (colour, bold and so on)
- * given a current state. The output buffer must be able to hold at least 57
- * bytes.
+ * given a current state.
  */
 static void
 grid_string_cells_code(const struct grid_cell *lastgc,
@@ -847,9 +911,8 @@ grid_string_cells(struct grid *gd, u_int px, u_int py, u_int nx,
 }
 
 /*
- * Duplicate a set of lines between two grids. If there aren't enough lines in
- * either source or destination, the number of lines is limited to the number
- * available.
+ * Duplicate a set of lines between two grids. Both source and destination
+ * should be big enough.
  */
 void
 grid_duplicate_lines(struct grid *dst, u_int dy, struct grid *src, u_int sy,
@@ -862,7 +925,7 @@ grid_duplicate_lines(struct grid *dst, u_int dy, struct grid *src, u_int sy,
 		ny = dst->hsize + dst->sy - dy;
 	if (sy + ny > src->hsize + src->sy)
 		ny = src->hsize + src->sy - sy;
-	grid_clear_lines(dst, dy, ny, 8);
+	grid_free_lines(dst, dy, ny);
 
 	for (yy = 0; yy < ny; yy++) {
 		srcl = &src->linedata[sy];
