@@ -24,8 +24,10 @@
 
 #include "tmux.h"
 
-void	screen_resize_x(struct screen *, u_int);
-void	screen_resize_y(struct screen *, u_int);
+static void	screen_resize_x(struct screen *, u_int);
+static void	screen_resize_y(struct screen *, u_int);
+
+static void	screen_reflow(struct screen *, u_int);
 
 /* Create a new screen. */
 void
@@ -38,9 +40,6 @@ screen_init(struct screen *s, u_int sx, u_int sy, u_int hlimit)
 	s->ccolour = xstrdup("");
 	s->tabs = NULL;
 
-	s->dirty = NULL;
-	s->dirtysize = 0;
-
 	screen_reinit(s);
 }
 
@@ -58,7 +57,7 @@ screen_reinit(struct screen *s)
 
 	screen_reset_tabs(s);
 
-	grid_clear_lines(s->grid, s->grid->hsize, s->grid->sy);
+	grid_clear_lines(s->grid, s->grid->hsize, s->grid->sy, 8);
 
 	screen_clear_selection(s);
 }
@@ -67,7 +66,6 @@ screen_reinit(struct screen *s)
 void
 screen_free(struct screen *s)
 {
-	free(s->dirty);
 	free(s->tabs);
 	free(s->title);
 	free(s->ccolour);
@@ -109,7 +107,7 @@ void
 screen_set_title(struct screen *s, const char *title)
 {
 	free(s->title);
-	s->title = xstrdup(title);
+	utf8_stravis(&s->title, title, VIS_OCTAL|VIS_CSTYLE|VIS_TAB|VIS_NL);
 }
 
 /* Resize screen. */
@@ -139,7 +137,7 @@ screen_resize(struct screen *s, u_int sx, u_int sy, int reflow)
 		screen_reflow(s, sx);
 }
 
-void
+static void
 screen_resize_x(struct screen *s, u_int sx)
 {
 	struct grid		*gd = s->grid;
@@ -161,7 +159,7 @@ screen_resize_x(struct screen *s, u_int sx)
 	gd->sx = sx;
 }
 
-void
+static void
 screen_resize_y(struct screen *s, u_int sy)
 {
 	struct grid	*gd = s->grid;
@@ -191,7 +189,8 @@ screen_resize_y(struct screen *s, u_int sy)
 		if (available > 0) {
 			if (available > needed)
 				available = needed;
-			grid_view_delete_lines(gd, oldy - available, available);
+			grid_view_delete_lines(gd, oldy - available, available,
+			    8);
 		}
 		needed -= available;
 
@@ -207,7 +206,7 @@ screen_resize_y(struct screen *s, u_int sy)
 		} else if (needed > 0 && available > 0) {
 			if (available > needed)
 				available = needed;
-			grid_view_delete_lines(gd, 0, available);
+			grid_view_delete_lines(gd, 0, available, 8);
 		}
 		s->cy -= needed;
 	}
@@ -221,8 +220,8 @@ screen_resize_y(struct screen *s, u_int sy)
 		needed = sy - oldy;
 
 		/*
-		 * Try to pull as much as possible out of scrolled history, if is
-		 * is enabled.
+		 * Try to pull as much as possible out of scrolled history, if
+		 * is is enabled.
 		 */
 		available = gd->hscrolled;
 		if (gd->flags & GRID_HISTORY && available > 0) {
@@ -255,6 +254,8 @@ screen_set_selection(struct screen *s, u_int sx, u_int sy,
 
 	memcpy(&sel->cell, gc, sizeof sel->cell);
 	sel->flag = 1;
+	sel->hidden = 0;
+
 	sel->rectflag = rectflag;
 
 	sel->sx = sx; sel->sy = sy;
@@ -268,9 +269,19 @@ screen_clear_selection(struct screen *s)
 	struct screen_sel	*sel = &s->sel;
 
 	sel->flag = 0;
+	sel->hidden = 0;
 	sel->lineflag = LINE_SEL_NONE;
 }
 
+/* Hide selection. */
+void
+screen_hide_selection(struct screen *s)
+{
+	struct screen_sel	*sel = &s->sel;
+
+	sel->hidden = 1;
+}
+
 /* Check if cell in selection. */
 int
 screen_check_selection(struct screen *s, u_int px, u_int py)
@@ -278,7 +289,7 @@ screen_check_selection(struct screen *s, u_int px, u_int py)
 	struct screen_sel	*sel = &s->sel;
 	u_int			 xx;
 
-	if (!sel->flag)
+	if (!sel->flag || sel->hidden)
 		return (0);
 
 	if (sel->rectflag) {
@@ -343,7 +354,7 @@ screen_check_selection(struct screen *s, u_int px, u_int py)
 				xx = sel->sx - 1;
 			else
 				xx = sel->sx;
-			if (py == sel->sy && px > xx)
+			if (py == sel->sy && (sel->sx == 0 || px > xx))
 				return (0);
 		} else {
 			/* starting line == ending line. */
@@ -369,8 +380,24 @@ screen_check_selection(struct screen *s, u_int px, u_int py)
 	return (1);
 }
 
-/* Reflow wrapped lines. */
+/* Get selected grid cell. */
 void
+screen_select_cell(struct screen *s, struct grid_cell *dst,
+    const struct grid_cell *src)
+{
+	if (!s->sel.flag || s->sel.hidden)
+		return;
+
+	memcpy(dst, &s->sel.cell, sizeof *dst);
+
+	utf8_copy(&dst->data, &src->data);
+	dst->attr = dst->attr & ~GRID_ATTR_CHARSET;
+	dst->attr |= src->attr & GRID_ATTR_CHARSET;
+	dst->flags = src->flags;
+}
+
+/* Reflow wrapped lines. */
+static void
 screen_reflow(struct screen *s, u_int new_x)
 {
 	struct grid	*old = s->grid;
