@@ -165,6 +165,10 @@ static const struct tty_default_key_raw tty_default_raw_keys[] = {
 	/* Focus tracking. */
 	{ "\033[I", KEYC_FOCUS_IN },
 	{ "\033[O", KEYC_FOCUS_OUT },
+
+	/* Paste keys. */
+	{ "\033[200~", KEYC_PASTE_START },
+	{ "\033[201~", KEYC_PASTE_END },
 };
 
 /* Default terminfo(5) keys. */
@@ -267,6 +271,7 @@ static const struct tty_default_key_code tty_default_code_keys[] = {
 	{ TTYC_KDC5, KEYC_DC|KEYC_CTRL|KEYC_XTERM },
 	{ TTYC_KDC6, KEYC_DC|KEYC_SHIFT|KEYC_CTRL|KEYC_XTERM },
 	{ TTYC_KDC7, KEYC_DC|KEYC_ESCAPE|KEYC_CTRL|KEYC_XTERM },
+	{ TTYC_KIND, KEYC_UP|KEYC_SHIFT|KEYC_XTERM },
 	{ TTYC_KDN2, KEYC_DOWN|KEYC_SHIFT|KEYC_XTERM },
 	{ TTYC_KDN3, KEYC_DOWN|KEYC_ESCAPE|KEYC_XTERM },
 	{ TTYC_KDN4, KEYC_DOWN|KEYC_SHIFT|KEYC_ESCAPE|KEYC_XTERM },
@@ -315,6 +320,7 @@ static const struct tty_default_key_code tty_default_code_keys[] = {
 	{ TTYC_KRIT5, KEYC_RIGHT|KEYC_CTRL|KEYC_XTERM },
 	{ TTYC_KRIT6, KEYC_RIGHT|KEYC_SHIFT|KEYC_CTRL|KEYC_XTERM },
 	{ TTYC_KRIT7, KEYC_RIGHT|KEYC_ESCAPE|KEYC_CTRL|KEYC_XTERM },
+	{ TTYC_KRI, KEYC_UP|KEYC_SHIFT|KEYC_XTERM },
 	{ TTYC_KUP2, KEYC_UP|KEYC_SHIFT|KEYC_XTERM },
 	{ TTYC_KUP3, KEYC_UP|KEYC_ESCAPE|KEYC_XTERM },
 	{ TTYC_KUP4, KEYC_UP|KEYC_SHIFT|KEYC_ESCAPE|KEYC_XTERM },
@@ -385,8 +391,9 @@ tty_keys_build(struct tty *tty)
 {
 	const struct tty_default_key_raw	*tdkr;
 	const struct tty_default_key_code	*tdkc;
-	u_int		 			 i;
-	const char				*s;
+	u_int		 			 i, size;
+	const char				*s, *value;
+	struct options_entry			*o;
 
 	if (tty->key_tree != NULL)
 		tty_keys_free(tty);
@@ -407,6 +414,15 @@ tty_keys_build(struct tty *tty)
 			tty_keys_add(tty, s, tdkc->key);
 
 	}
+
+	o = options_get(global_options, "user-keys");
+	if (o != NULL && options_array_size(o, &size) != -1) {
+		for (i = 0; i < size; i++) {
+			value = options_array_get(o, i);
+			if (value != NULL)
+				tty_keys_add(tty, value, KEYC_USER + i);
+		}
+	}
 }
 
 /* Free the entire key tree. */
@@ -579,7 +595,17 @@ tty_keys_next(struct tty *tty)
 	}
 
 first_key:
-	/* Handle keys starting with escape. */
+	/* Try to lookup complete key. */
+	n = tty_keys_next1(tty, buf, len, &key, &size, expired);
+	if (n == 0)	/* found */
+		goto complete_key;
+	if (n == 1)
+		goto partial_key;
+
+	/*
+	 * If not a complete key, look for key with an escape prefix (meta
+	 * modifier).
+	 */
 	if (*buf == '\033') {
 		/* Look for a key without the escape. */
 		n = tty_keys_next1(tty, buf + 1, len - 1, &key, &size, expired);
@@ -604,13 +630,6 @@ first_key:
 			goto partial_key;
 	}
 
-	/* Try to lookup key. */
-	n = tty_keys_next1(tty, buf, len, &key, &size, expired);
-	if (n == 0)	/* found */
-		goto complete_key;
-	if (n == 1)
-		goto partial_key;
-
 	/*
 	 * At this point, we know the key is not partial (with or without
 	 * escape). So pass it through even if the timer has not expired.
