@@ -387,6 +387,23 @@ window_destroy(struct window *w)
 	free(w);
 }
 
+int
+window_pane_destroy_ready(struct window_pane *wp)
+{
+	int	n;
+
+	if (wp->pipe_fd != -1) {
+		if (EVBUFFER_LENGTH(wp->pipe_event->output) != 0)
+			return (0);
+		if (ioctl(wp->fd, FIONREAD, &n) != -1 && n > 0)
+			return (0);
+	}
+
+	if (~wp->flags & PANE_EXITED)
+		return (0);
+	return (1);
+}
+
 void
 window_add_ref(struct window *w, const char *from)
 {
@@ -408,7 +425,7 @@ void
 window_set_name(struct window *w, const char *new_name)
 {
 	free(w->name);
-	w->name = xstrdup(new_name);
+	utf8_stravis(&w->name, new_name, VIS_OCTAL|VIS_CSTYLE|VIS_TAB|VIS_NL);
 	notify_window("window-renamed", w);
 }
 
@@ -615,6 +632,8 @@ window_add_pane(struct window *w, struct window_pane *other, int before,
 void
 window_lost_pane(struct window *w, struct window_pane *wp)
 {
+	log_debug("%s: @%u pane %%%u", __func__, w->id, wp->id);
+
 	if (wp == marked_pane.wp)
 		server_clear_marked();
 
@@ -800,8 +819,8 @@ window_pane_create(struct window *w, u_int sx, u_int sy, u_int hlimit)
 	wp->xoff = 0;
 	wp->yoff = 0;
 
-	wp->sx = sx;
-	wp->sy = sy;
+	wp->sx = wp->osx = sx;
+	wp->sy = wp->osx = sy;
 
 	wp->pipe_fd = -1;
 	wp->pipe_off = 0;
@@ -828,6 +847,7 @@ static void
 window_pane_destroy(struct window_pane *wp)
 {
 	window_pane_reset_mode(wp);
+	free(wp->searchstr);
 
 	if (wp->fd != -1) {
 #ifdef HAVE_UTEMPTER
@@ -873,6 +893,7 @@ window_pane_spawn(struct window_pane *wp, int argc, char **argv,
 	char		 s[32];
 #endif
 	int		 i;
+	sigset_t	 set, oldset;
 
 	if (wp->fd != -1) {
 		bufferevent_free(wp->event);
@@ -902,14 +923,21 @@ window_pane_spawn(struct window_pane *wp, int argc, char **argv,
 	ws.ws_col = screen_size_x(&wp->base);
 	ws.ws_row = screen_size_y(&wp->base);
 
-	wp->pid = fdforkpty(ptm_fd, &wp->fd, wp->tty, NULL, &ws);
-	switch (wp->pid) {
+	sigfillset(&set);
+	sigprocmask(SIG_BLOCK, &set, &oldset);
+	switch (wp->pid = fdforkpty(ptm_fd, &wp->fd, wp->tty, NULL, &ws)) {
 	case -1:
 		wp->fd = -1;
+
 		xasprintf(cause, "%s: %s", cmd, strerror(errno));
 		free(cmd);
+
+		sigprocmask(SIG_SETMASK, &oldset, NULL);
 		return (-1);
 	case 0:
+		proc_clear_signals(server_proc, 1);
+		sigprocmask(SIG_SETMASK, &oldset, NULL);
+
 		if (chdir(wp->cwd) != 0) {
 			if ((home = find_home()) == NULL || chdir(home) != 0)
 				chdir("/");
@@ -926,6 +954,7 @@ window_pane_spawn(struct window_pane *wp, int argc, char **argv,
 		if (tcsetattr(STDIN_FILENO, TCSANOW, &tio2) != 0)
 			fatal("tcgetattr failed");
 
+		log_close();
 		closefrom(STDERR_FILENO + 1);
 
 		if (path != NULL)
@@ -933,9 +962,6 @@ window_pane_spawn(struct window_pane *wp, int argc, char **argv,
 		environ_set(env, "TMUX_PANE", "%%%u", wp->id);
 		environ_push(env);
 
-		clear_signals(1);
-		log_close();
-
 		setenv("SHELL", wp->shell, 1);
 		ptr = strrchr(wp->shell, '/');
 
@@ -974,6 +1000,7 @@ window_pane_spawn(struct window_pane *wp, int argc, char **argv,
 	kill(getpid(), SIGCHLD);
 #endif
 
+	sigprocmask(SIG_SETMASK, &oldset, NULL);
 	setblocking(wp->fd, 0);
 
 	wp->event = bufferevent_new(wp->fd, window_pane_read_callback, NULL,
@@ -1013,7 +1040,11 @@ window_pane_error_callback(__unused struct bufferevent *bufev,
 {
 	struct window_pane *wp = data;
 
-	server_destroy_pane(wp, 1);
+	log_debug("%%%u error", wp->id);
+	wp->flags |= PANE_EXITED;
+
+	if (window_pane_destroy_ready(wp))
+		server_destroy_pane(wp, 1);
 }
 
 void
@@ -1185,7 +1216,8 @@ window_pane_mode_timer(__unused int fd, __unused short events, void *arg)
 }
 
 int
-window_pane_set_mode(struct window_pane *wp, const struct window_mode *mode)
+window_pane_set_mode(struct window_pane *wp, const struct window_mode *mode,
+    struct cmd_find_state *fs, struct args *args)
 {
 	struct screen	*s;
 	struct timeval	 tv = { .tv_sec = 10 };
@@ -1198,7 +1230,7 @@ window_pane_set_mode(struct window_pane *wp, const struct window_mode *mode)
 	evtimer_set(&wp->modetimer, window_pane_mode_timer, wp);
 	evtimer_add(&wp->modetimer, &tv);
 
-	if ((s = wp->mode->init(wp)) != NULL)
+	if ((s = wp->mode->init(wp, fs, args)) != NULL)
 		wp->screen = s;
 	wp->flags |= (PANE_REDRAW|PANE_CHANGED);
 
@@ -1238,7 +1270,7 @@ window_pane_key(struct window_pane *wp, struct client *c, struct session *s,
 	if (wp->mode != NULL) {
 		wp->modelast = time(NULL);
 		if (wp->mode->key != NULL)
-			wp->mode->key(wp, c, s, key, m);
+			wp->mode->key(wp, c, s, (key & ~KEYC_XTERM), m);
 		return;
 	}
 
@@ -1262,49 +1294,42 @@ window_pane_key(struct window_pane *wp, struct client *c, struct session *s,
 }
 
 int
-window_pane_outside(struct window_pane *wp)
+window_pane_visible(struct window_pane *wp)
 {
 	struct window	*w = wp->window;
 
-	if (wp->xoff >= w->sx || wp->yoff >= w->sy)
-		return (1);
-	if (wp->xoff + wp->sx > w->sx || wp->yoff + wp->sy > w->sy)
-		return (1);
-	return (0);
-}
-
-int
-window_pane_visible(struct window_pane *wp)
-{
 	if (wp->layout_cell == NULL)
 		return (0);
-	return (!window_pane_outside(wp));
+
+	if (wp->xoff >= w->sx || wp->yoff >= w->sy)
+		return (0);
+	if (wp->xoff + wp->sx > w->sx || wp->yoff + wp->sy > w->sy)
+		return (0);
+	return (1);
 }
 
-char *
-window_pane_search(struct window_pane *wp, const char *searchstr,
-    u_int *lineno)
+u_int
+window_pane_search(struct window_pane *wp, const char *searchstr)
 {
 	struct screen	*s = &wp->base;
-	char		*newsearchstr, *line, *msg;
+	char		*newsearchstr, *line;
 	u_int		 i;
 
-	msg = NULL;
 	xasprintf(&newsearchstr, "*%s*", searchstr);
 
 	for (i = 0; i < screen_size_y(s); i++) {
 		line = grid_view_string_cells(s->grid, 0, i, screen_size_x(s));
 		if (fnmatch(newsearchstr, line, 0) == 0) {
-			msg = line;
-			if (lineno != NULL)
-				*lineno = i;
+			free(line);
 			break;
 		}
 		free(line);
 	}
 
 	free(newsearchstr);
-	return (msg);
+	if (i == screen_size_y(s))
+		return (0);
+	return (i + 1);
 }
 
 /* Get MRU pane from a list. */
